#!/usr/local/bin/python
from MythTV import MythDB, MythLog, MythTV, MythVideo, Program, Job
from socket import gethostname
import sys, getopt, re, os, time, logging, pprint

#Usage: mythvidexport.py <--chanid <channel id>> <--starttime <start time>> [options]
#		--fformat <see directory/file name options>
#		--tformat <see directory/file name options>
#	moves a single file over to MythVideo with optional directory format, format is discarded after use

#Usage: mythvidexport.py <job id>
#		WARNING: for internal use by job queue only!
#	moves a single file, grabbing necessary information from job queue table

#Usage: mythvidexport.py <options>
#		--setformat <see directory/file name options>
#			stored directory format in database for future use
#		--help
#		--helpformat
#			lengthy discription of directory formatting


class VIDEO:
	mythdb = None
	dbconn = None
	job = None
	jobhost = None
	chanid = None
	rtime = None
	source = None
	srchost = None
	srcdb = None
	dest = None
	desthost = None
	destdb = None
	prog = None
	fmt = None
	log = None
	epstr = None

	season= None
	episode = None
	viddata = {}
	cast = None
	genre = None
	country = None

	def __init__(self, *inp):
		if len(inp) == 1:
			self.job = Job(inp[0]);
			self.chanid = self.job.chanid
			self.rtime = "%04d%02d%02d%02d%02d%02d" % self.job.starttime.timetuple()[0:6]
			self.jobhost = self.job.host
			self.job.setStatus(3)
		elif len(inp) == 2:
			self.chanid = inp[0]
			self.rtime = inp[1]
		else:
			sys.exit(2)
		self.get_source()
		self.log = MythLog(logging.CRITICAL, '#%(levelname)s - %(message)s', 'MythTV')

	def dbconnect(self):
		self.mythdb = MythDB()
		self.dbconn = self.mythdb.db

	def get_source(self):
		if not self.dbconn:
			self.dbconnect()
		if not self.jobhost:
			self.jobhost = gethostname()

		mythinst = MythTV()
		self.prog = mythinst.getRecording(self.chanid,int(self.rtime))

		self.srcdb = self.prog.filename
		self.srchost = self.prog.hostname

		self.source = mythinst.getCheckfile(self.prog)

	def find_ttvdb(self):
		if os.access('/usr/local/share/mythtv/mythvideo/scripts/ttvdb.py',os.F_OK):
			self.ttvdb = '/usr/local/share/mythtv/mythvideo/scripts/ttvdb.py'
		elif os.access('/usr/share/mythtv/mythvideo/scripts/ttvdb.py',os.F_OK):
			self.ttvdb = '/usr/local/share/mythtv/mythvideo/scripts/ttvdb.py'
		else:
			self.log.Msg(logging.CRITICAL, "Could not find ttvdb.py")
			self.exit(2)

	def get_meta(self, use_ttvdb):
		regex = re.compile('S(?P<season>[0-9]*)E(?P<episode>[0-9]*)')
		match = None
		if use_ttvdb == 0:
			match = regex.search(self.epstr)
		elif use_ttvdb in (1,2):
			self.find_ttvdb()
			fp = os.popen("%s -N \"%s\" \"%s\"" % (self.ttvdb,self.prog.title,self.prog.subtitle),"r")
			match = regex.search(fp.read())
			fp.close()
		if match is None:
			if self.job:
				self.job.setComment("TTVDB.py failed to get season/episode numbers")
				self.job.setStatus(304)
				self.log.Msg(logging.CRITICAL, "TTVDB.py failed to get season/episode numbers")
			else:
				print("TTVDB.py failed to get season/episode numbers")
			sys.exit(2)
		self.season = int(match.group('season'))
		self.episode = int(match.group('episode'))

		if use_ttvdb == 2:
			print("calling '%s -mD '%s' %d %d'" % (self.ttvdb,self.prog.title,self.season,self.episode))
			fp = os.popen("%s -mD '%s' %d %d" % (self.ttvdb,self.prog.title,self.season,self.episode))
			time.sleep(2)
			res = fp.read()
			fp.close()
			ttvdbdat = {}
			if len(res) == 0:
				self.get_metadata(1)
				return
			for point in res.split('\n')[:-1]:
				key,dat = point.split(':',1)
				ttvdbdat[key] = dat
			self.viddata['director'] = ttvdbdat['Director']
			self.viddata['plot'] = ttvdbdat['Plot']
			self.viddata['year'] = ttvdbdat['Year']
			self.viddata['length'] = ttvdbdat['Runtime']
			self.viddata['userrating'] = ttvdbdat['UserRating']
			self.cast = tuple(ttvdbdat['Cast'].split(', '))
			self.genre = tuple(ttvdbdat['Genres'].split(', '))
			self.country = ()
		else:
			self.viddata['director'] = self.mythdb.getCast(self.chanid, int(self.rtime), roles='director')
			if len(self.viddata['director']) == 0:
				self.viddata['director'] = 'NULL'
			else:
				self.viddata['director'] = self.viddata['director'][0]
			self.viddata['plot'] = self.prog.description
			self.viddata['year'] = self.prog.year
			self.viddata['length'] = str(int((self.prog.recendts-self.prog.recstartts).seconds/60))
			self.cast = self.mythdb.getCast(self.chanid, int(self.rtime), roles=('actor','guest_star','host','commentator','guest'))
			self.genre = ()
			self.country = ()

		self.viddata['showlevel'] = 1
		self.viddata['rating'] = 'NR'
		self.viddata['inetref'] = '00000000'
		self.viddata['coverfile'] = 'No Cover'

	def get_dest(self):
		#process self.fmt and generate self.dest
		#not programmed yet... just use my own naming scheme
		ext = self.source[self.source.rfind('.'):]
		#replace with sequence of str-replace for the formatted destination
		subpath = self.process_fmt(self.ffmt) + ext

		self.desthost = self.jobhost
		cursor = self.dbconn.cursor()
		reslen = cursor.execute("SELECT dirname FROM storagegroup WHERE hostname='%s' and groupname='Videos'" % self.desthost)
		if reslen:
			self.dest = cursor.fetchone()[0]+'/'+subpath
			self.destdb = subpath
		else:
			self.dest = self.mythdb.getSetting("VideoStartupDir",hostname=self.jobhost)+'/'+subpath
			self.destdb = self.dest


		tmppath = self.dest[0:self.dest.rfind('/')] 
		if not os.access(tmppath,os.F_OK):
			os.makedirs(tmppath)

	def set_tfmt(self, fmt):
		self.tfmt = fmt
	def set_ffmt(self, fmt):
		self.ffmt = fmt

	def get_fmt(self):
		self.ffmt = self.mythdb.getSetting('mythvideo.exportfmt')
		if not self.ffmt:
			self.ffmt = 'TV/%TITLE%/Season %SEASON%/%SEASON%x%EPISODE% - %SUBTITLE%'
		self.tfmt = self.mythdb.getSetting('mythvideo.titlefmt')
		if not self.tfmt:
			self.tfmt = '%SEASON%x%EPISODE% - %SUBTITLE%'

	def process_fmt(self, fmt):
		fmt.replace('%TITLE%',self.prog.title)
		fmt.replace('%SUBTITLE%',self.prog.subtitle)
		fmt.replace('%SEASON%',"%d" % self.season)
		fmt.replace('%EPISODE%',"%02d" % self.episode)
		fmt.replace('%YEAR%',self.viddata['year'])
		fmt.replace('%DIRECTOR%',self.viddata['director'])
#		fmt.replace('%CARDID%',self.prog.cardid)
#		fmt.replace('%CARDNAME%',self.prog.cardid)
#		fmt.replace('%SOURCEID%',self.prog.cardid)
#		fmt.replace('%SOURCENAME%',self.prog.cardid)
		fmt.replace('%HOSTNAME%',self.prog.hostname)
		fmt.replace('%STORAGEGROUP%',self.prog.storagegroup)
#		fmt.replace('%CHANNUM%',self.prog.channum)
#		fmt.replace('%CHANNAME%',self.prog.cardid)
		if len(self.genre):
			fmt.replace('%GENRE%',self.genre[0])
		else:
			fmt.replace('%GENRE%','')
#		if len(self.country):
#			fmt.replace('%COUNTRY%',self.country[0])
#		else:
#			fmt.replace('%COUNTRY%','')
		return fmt

	def copy(self):
		cursor = None
		fp = None
		srcsize = None
		dtime = None
		if self.fmt is None:
			self.get_fmt()
		self.get_dest()
		if self.desthost != self.jobhost:
			if self.job:
				self.job.setComment("Job must be run on destination host")
				self.log.Msg(logging.CRITICAL, 'MythVidExport failed - must be run on destination host')
				self.job.setStatus(304)
			else:
				print("ERROR: job must be run on destionation host")
			sys.exit(2)

		stime = time.time()
		print("Copying %s to %s..." % (self.source,self.dest))
		fp = os.popen("cp \"%s\" \"%s\"" % (self.source,self.dest))
#		if self.srchost == self.jobhost:
#			fp = os.popen("cp '%s' '%s'" % (self.source,self.dest))
#		else:
#			fp = os.popen("scp '%s:%s' '%s'" % (self.srchost, self.source, self.dest))
		srcsize = self.prog.filesize
		destsize = [0,0,0,0,0,0,0,0,0,0]
		if self.job:
			self.job.setStatus(4)
		while srcsize > destsize[9]:
			time.sleep(4)
			destsize.append(os.stat(self.dest)[6])
			rem = srcsize-destsize[10]
			if destsize[0]:
				dtime = 40
			else:
				dtime = int(time.time()-stime)
			rate = (destsize[10]-destsize.pop(0))/dtime
			remt = rem/rate
			if self.job:
				self.job.setComment("%02d%% complete - %s seconds remaining" % (destsize[9]*100/srcsize, remt))
		fp.close()
		if self.job:
			self.job.setComment("Complete - %d seconds elapsed" % (int(time.time()-stime)))
			self.job.setStatus(256)


	def write_meta(self):
		mythvid = MythVideo()
		mythtv = MythTV()
		pp = pprint.PrettyPrinter(indent=4)
		self.viddata['title'] = self.process_fmt(self.tfmt)
		self.viddata['filename'] = self.destdb
		self.viddata['host'] = self.desthost


		pp.pprint(self.viddata)
		intid = mythvid.getMetadataId(self.destdb)
		if intid:
			mythvid.setMetadata(self.viddata,intid)
		else:
			intid = mythvid.setMetadata(self.viddata)
		print("Metadata written to intid %d" % intid)
		pp.pprint(self.viddata)

		print("With cast:")
		pp.pprint(self.cast)
		for name in self.cast:
			mythvid.setCast(name, intid)
		print("With genre:")
		pp.pprint(self.genre)
#		for genre in self.genre:
#			mythvid.setGenre(genre, intid)

		if self.job:
			self.job.setStatus(272)

def usage():
	print("mythvidexport.py [options] [--chanid=<chanid> --starttime=<starttime> | <jobid>]")
	print("        This script can be run by specifing the channel and start time directly")
	print("            or by specifing the ID of a job in jobqueue")
	print("")
	print("        Run from the command line through the former:")
	print("            mythvidexport.py --chanid=1002 --starttime=200907010000")
        print("        Or from a user script through the latter:")
        print("            mythvidexport.py %JOBID%")
	print("")
	print("    Options:")
	print("        -h/--help and -f/--helpformat:")
	print("             return this help, or a listing of available formatting strings")
	print("        --fformat='<string>' and --tformat='<string>':")
	print("             override the stored formatting string in the database")
        print("             if no recording is specified, store format string to the database")
	print("        --episode=<string>:")
	print("             specify the episode number using the format S00E00")
	print("             removes the need for ttvdb.py to perform this lookup")
	print("        --ttvdb:")
	print("             use ttvdb.py to grab metadata rather than using available data")
	print("                 from listings source")

def usage_format():
	print("The default strings are:")
	print("    fformat: 'TV/%TITLE%/Season %SEASON%/%SEASON%x%EPISODE% - %SUBTITLE%'")
	print("    tformat: '%SEASON%x%EPISODE% - %SUBTITLE%'")
	print("")
	print("Available strings:")
	print("    %TITLE%:         series title")
	print("    %SUBTITLE%:      episode title")
	print("    %SEASON%:        season number")
	print("    %EPISODE%:       episode number, padded to 2 digits")
	print("    %YEAR%:          year")
	print("    %DIRECTOR%:      director")
#	print("    %CARDID%:        ID of tuner card used to record show")
#	print("    %CARDNAME%:      name of tuner card used to record show")
#	print("    %SOURCEID%:      ID of video source used to record show")
#	print("    %SOURCENAME%:    name of video source used to record show")
        print("    %HOSTNAME%:      backend used to record show")
	print("    %STORAGEGROUP%:  storage group containing recorded show")
#	print("    %CHANNUM%:       ID of channel used to record show")
#	print("    %CHANNAME%:      name of channel used to record show")
	print("    %GENRE%:         first genre listed for recording")
#	print("    %COUNTRY%:       first country listed for recording")

def main():
	fformat = None
	tformat = None
	jobid = None
	chanid = None
	rtime = None
	episode = None
	ttvdb = 1
	skip = False
	try:
		opts, args = getopt.getopt(sys.argv[1:], "hf", ["help","helpformat","fformat=g","tformat=","chanid=","starttime=","episode=","ttvdb","skip"])
	except getopt.GetoptError, err:
		print(str(err))
		usage()
		sys.exit(2)
	for opt, arg in opts:
		if opt in ("-h", "--help"):
			usage()
			sys.exit()
		elif opt in ('-f', '--helpformat'):
			usage_format()
			sys.exit()
		elif opt == '--tformat':
			tformat = arg
		elif opt == '--fformat':
			fformat = arg
		elif opt == '--chanid':
			chanid = int(arg)
		elif opt == '--starttime':
			rtime = int(arg)
		elif opt == '--episode':
			ttvdb = 0
			episode = arg
		elif opt == '--ttvdb':
			ttvdb = 2
		elif opt == '--skip':
			skip = True
		
	if len(args):
		jobid = int(args[0])

	if chanid and rtime:
		export = VIDEO(chanid,rtime)
	elif jobid:
		export = VIDEO(jobid)
	else:
		if fformat:
			mythdb = MythDB()
			mythdb.setSetting('mythvideo.exportfmt', fmt)
		if tformat:
			mythdb = MythDB()
			mythdb.setSetting('mythvideo.titlefmt', fmt)
		sys.exit()
	else:
		usage()
		sys.exit(2)

	if format:
		export.set_fmt(format)
	if episode:
		export.epstr = episode
	export.get_meta(ttvdb)
	if skip:
		export.get_dest()
	else:
		export.copy()
	export.write_meta()

if __name__ == "__main__":
	main()
